@@ -15,12 +15,12 @@ import hashlib
 import re
 from functools import lru_cache
 
-from .helper import Har2TreeError, Har2TreeLogAdapter
+from .helper import Har2TreeError, Har2TreeLogAdapter, make_hhhash, HHHashError
 
 import filetype  # type: ignore
 from bs4 import BeautifulSoup
 from ete3 import TreeNode  # type: ignore
-from publicsuffix2 import PublicSuffixList, fetch  # type: ignore
+from publicsuffixlist import PublicSuffixList  # type: ignore
 from w3lib.html import strip_html5_whitespace
 from w3lib.url import canonicalize_url, safe_url_string
 
@@ -28,13 +28,8 @@ from w3lib.url import canonicalize_url, safe_url_string
 @lru_cache(64)
 def get_public_suffix_list() -> PublicSuffixList:
     # Initialize Public Suffix List
-    try:
-        psl_file = fetch()
-        psl = PublicSuffixList(psl_file=psl_file)
-    except Exception as e:
-        logging.getLogger(__name__).warning(f'Unable to fetch the PublicSuffixList: {e}')
-        psl = PublicSuffixList()
-    return psl
+    # TODO (?): fetch the list
+    return PublicSuffixList()
 
 
 class HarTreeNode(TreeNode):
@@ -138,7 +133,10 @@ class URLNode(HarTreeNode):
             # so we need that unless we want to change the JS
             self.add_feature('hostname', str(Path(self.url_split.path).parent))
         else:
-            self.add_feature('hostname', self.url_split.hostname)
+            if self.url_split.hostname:
+                self.add_feature('hostname', self.url_split.hostname)
+            else:
+                self.add_feature('hostname', self.name)
 
         if not self.hostname:
             self.logger.warning(f'Something is broken in that node: {har_entry}')
@@ -151,7 +149,7 @@ class URLNode(HarTreeNode):
             pass
 
         if not hasattr(self, 'hostname_is_ip') and not hasattr(self, 'file_on_disk'):
-            tld = get_public_suffix_list().get_tld(self.hostname, strict=True)
+            tld = get_public_suffix_list().publicsuffix(self.hostname)
             if tld:
                 self.add_feature('known_tld', tld)
             else:
@@ -189,7 +187,7 @@ class URLNode(HarTreeNode):
                             try:
                                 posted_data = posted_data.decode()
                             except Exception:
-                                self.logger.warning(f'Expected urlencoded, got garbage: {posted_data!r}')
+                                self.logger.debug(f'Expected urlencoded, got garbage: {posted_data!r}')
                         if isinstance(posted_data, str):
                             posted_data = unquote_plus(posted_data)
                     elif (self.request['postData']['mimeType'].startswith('application/json')
@@ -209,7 +207,7 @@ class URLNode(HarTreeNode):
                             try:
                                 posted_data = json.loads(posted_data)
                             except Exception:
-                                self.logger.warning(f"Expected json, got garbage: {self.request['postData']['mimeType']} - {posted_data[:20]!r}[...]")
+                                self.logger.debug(f"Expected json, got garbage: {self.request['postData']['mimeType']} - {posted_data[:20]!r}[...]")
 
                     elif self.request['postData']['mimeType'].startswith('multipart/form-data'):
                         # FIXME multipart content (similar to email). Not totally sure what do do with it tight now.
@@ -245,8 +243,12 @@ class URLNode(HarTreeNode):
                 self.add_feature('posted_data', posted_data)
 
         self.add_feature('response', har_entry['response'])
+        try:
+            self.add_feature('hhhash', make_hhhash(self.response))
+        except HHHashError as e:
+            self.logger.warning(e)
 
-        self.add_feature('response_cookie', har_entry['response']['cookies'])
+        self.add_feature('response_cookie', self.response['cookies'])
         if self.response_cookie:
             self.add_feature('set_third_party_cookies', False)
             # https://developer.mozilla.org/en-US/docs/Web/HTTP/headers/Set-Cookie
@@ -279,24 +281,29 @@ class URLNode(HarTreeNode):
             for cookie in self.request_cookie:
                 self.cookies_sent[f'{cookie["name"]}={cookie["value"]}'] = []
 
-        if not har_entry['response']['content'].get('text') or har_entry['response']['content']['text'] == '':
+        if not self.response['content'].get('text') or self.response['content']['text'] == '':
             # If the content of the response is empty, skip.
             self.add_feature('empty_response', True)
         else:
             self.add_feature('empty_response', False)
-            if har_entry['response']['content'].get('encoding') == 'base64':
-                self.add_feature('body', BytesIO(b64decode(har_entry['response']['content']['text'])))
+            if self.response['content'].get('encoding') == 'base64':
+                self.add_feature('body', BytesIO(b64decode(self.response['content']['text'])))
             else:
-                self.add_feature('body', BytesIO(har_entry['response']['content']['text'].encode()))
+                self.add_feature('body', BytesIO(self.response['content']['text'].encode()))
+
             self.add_feature('body_hash', hashlib.sha512(self.body.getvalue()).hexdigest())
-            if har_entry['response']['content']['mimeType']:
-                self.add_feature('mimetype', har_entry['response']['content']['mimeType'].lower())
-            else:
-                kind = filetype.guess(self.body.getvalue())
-                if kind:
+            if self.response['content']['mimeType']:
+                mt = self.response['content']['mimeType'].lower()
+                if mt not in ["application/octet-stream", "x-unknown"]:
+                    self.add_feature('mimetype', mt)
+
+            if not hasattr(self, 'mimetype'):
+                # try to guess something better
+                if kind := filetype.guess(self.body.getvalue()):
                     self.add_feature('mimetype', kind.mime)
-                else:
-                    self.add_feature('mimetype', '')
+
+            if not hasattr(self, 'mimetype'):
+                self.add_feature('mimetype', '')
 
             external_ressources, embedded_ressources = find_external_ressources(self.body.getvalue(), self.name, all_requests)
             self.add_feature('external_ressources', external_ressources)
@@ -366,9 +373,9 @@ class URLNode(HarTreeNode):
                 har_entry['_securityDetails']['validTo'] = datetime.fromtimestamp(har_entry['_securityDetails']['validTo'])
             self.add_feature('security_details', har_entry['_securityDetails'])
 
-        if har_entry['response']['redirectURL']:
+        if self.response['redirectURL']:
             self.add_feature('redirect', True)
-            redirect_url = har_entry['response']['redirectURL']
+            redirect_url = self.response['redirectURL']
             # Rebuild the redirect URL so it matches the entry that sould be in all_requests
             redirect_url = rebuild_url(self.name, redirect_url, all_requests)
             # At this point, we should have a URL available in all_requests...
@@ -377,10 +384,10 @@ class URLNode(HarTreeNode):
             else:
                 # ..... Or not. Unable to find a URL for this redirect
                 self.add_feature('redirect_to_nothing', True)
-                self.add_feature('redirect_url', har_entry['response']['redirectURL'])
+                self.add_feature('redirect_url', self.response['redirectURL'])
                 self.logger.warning('Unable to find that URL: {original_url} - {original_redirect} - {modified_redirect}'.format(
                     original_url=self.name,
-                    original_redirect=har_entry['response']['redirectURL'],
+                    original_redirect=self.response['redirectURL'],
                     modified_redirect=redirect_url))
 
     @property
@@ -553,6 +560,10 @@ class HostNode(HarTreeNode):
         if not self.name:
             # Only used when initializing the root node
             self.add_feature('name', url.hostname)
+
+        if hasattr(url, 'hostname_is_ip') and url.hostname_is_ip:
+            self.add_feature('hostname_is_ip', True)
+
         self.urls.append(url)
 
         # Add to URLNode a reference to the HostNode UUID
